//********************************************************************
//*** ResourcePool/SQL/SQLExecute.t.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLExecute.t.cc,v 1.7 2005/01/06 16:27:56 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

using namespace fatalmind;

std::auto_ptr<RP> _rp;

void test_init()
{
	if (_rp.get() == 0) {
		_rp.reset(new RP(getFactory()));
	}
}

void test_SQLExecute_init() {
	test_init();
	init(*_rp);
}

void test_SQLExecute_one()
{
	RP::Command::SQLExecute cmd("insert into hirsch (id) values (?)");

	cmd.bind(0, 4);
	_rp->execute(cmd);
}

void test_SQLExecute_two()
{
	RP::Command::SQLExecute cmd("insert into hirsch (id, ukn) values (?, ?)");

	cmd.bind(0, 5);
	cmd.bind<std::string>(1, "hirsch");
	_rp->execute(cmd);
}

void test_SQLExecute_three()
{
	RP p(getFactory());
	RP::Command::SQLExecute cmd("insert into hirsch (id, ukn) values (?, ?)");

	cmd.bind(0, 6);
	cmd.bind<std::string>(1, "hirsch");
	cmd.execute(p.get());
	cmd.bind(0, 7);
	cmd.execute(p.get());
}

void test_perf_SQLExecute_insert()
{
	clear(*_rp);

	for (int i = 0; i < 600; ++i) {
		RP::Command::SQLExecute inscmd("insert into hirsch (id, ukn) values (?, ?)");
		inscmd.bind(0, i);
		inscmd.bind<std::string>(1, "hohoho ' kasdfjs \" sdf ");
		_rp->execute(inscmd);
	}
}

void test_perf_SQLExecute_insert_outer()
{
	clear(*_rp);

	RP::Command::SQLExecute inscmd("insert into hirsch (id, ukn) values (?, ?)");
	inscmd.bind<std::string>(1, "hohoho ' kasdfjs \" sdf ");
	for (int i = 0; i < 600; ++i) {
		inscmd.bind(0, i);
		_rp->execute(inscmd);
	}
}

void test_perf_SQLExecute_update_outer()
{
	RP::Command::SQLExecute inscmd("update hirsch set ukn = ? where id = ?");
	inscmd.bind<std::string>(0, "updated");
	for (int i = 0; i < 600; ++i) {
		inscmd.bind(1, i);
		_rp->execute(inscmd);
	}
}

void test_SQLExecute_fail() {
	clear(*_rp);

	RP::Command::SQLExecute cmd("insert into hirsch (id, ukn) values (?, ?)");
	cmd.bind(0, 4);
	cmd.bind<std::string>(1, "rudi");

	_rp->execute(cmd);

	try {
		_rp->execute(cmd);
		assertNotReach();
	} catch (...) { // TODO:: this is wrong
	}
}

void test_SQLExecute_cleanup() {
	drop(*_rp);
}


