//********************************************************************
//*** ResourcePool/SQL/SQLSelect.hh
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: SQLSelect.hh,v 1.5 2005/01/08 20:02:18 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_SQL_SQLSELECT_HH
#define RESOURCEPOOL_SQL_SQLSELECT_HH

#include <string>

namespace fatalmind {

/**
 * This class provides the ability to perform a SQL query which returns an arbitary number of rows.
 */
class SQLSelect {
	public:
//		SQLSelect(const str::string&);
        /**
         * This method is to be overloaded by the user.
         * It's called every time a row has been fetched.
         * The variables you have bound are filled accordingly.
         *
         * There are are no special callbacks available to be called
         * if the last row has been fetched. Actually when process() is
         * called you don't know if more records will be fetched or if
         * this was the last record.
         *
         * If you need to do something after the last record has been fetched
         * it is considerd to be the best practice to override the execute()
         * method which calls the super::execute() and does what ever you want
         * afterwards.
         */
        virtual bool process() = 0;
};

}
#endif
