//********************************************************************
//*** ResourcePool/oracle/OracleException.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: OracleException.cc,v 1.16 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of 
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef RESOURCEPOOL_ORACLE_ORACLEEXCEPTION_HH
#include "OracleException.hh"
#endif

namespace fatalmind {

OracleException::OracleException(const OracleException& rho) 
: Exception(rho)
, SQLException(rho)
, _errorcode(rho._errorcode)
{
}

OracleException::OracleException(
	  const std::string& msg
	, const int errorcode
	, const std::string& sqlstm
):Exception(msg)
, SQLException(msg, mapORAtoXOPEN(errorcode), sqlstm, true)
, _errorcode(errorcode)
{
}

const char* OracleException::mapORAtoXOPEN(const int errorcode)
{
	switch(errorcode) {
		case 1400: 
		case 1401:
		case 1407:
		case 1: return "23000"; break; // integrity constraint violation
		case 1004:
		case 1005: return "08004";
		case 1756: return "07000";
		case 1006: 
		case 1007: 
		case 1062:
		case 24374:
		case 1008: return "07001"; 
		case 1017: return "28000"; // invalid authorization
		case 1041: return "42000"; // syntax error
		case 1402: return "44000"; // with check
		case 1406: return "01004"; // trancated
		case 1422: return "22000";
		case 999999: return "02000";
	}
	if (errorcode >= 900 && errorcode <= 999) {
		return "07000"; // dynamic sql error
	}
	return "08000"; // connection error
}

int OracleException::getErrorCode() const
{
	return _errorcode;
}

NFOOracleException::NFOOracleException(
	  const std::string& msg
	, const int errorcode
	, const std::string& sqlstm
):Exception(msg)
, SQLException(msg, mapORAtoXOPEN(errorcode), sqlstm, false)
, OracleException(msg, errorcode, sqlstm)
, NoFailoverException(msg)
{
}

} // namespace fatalmind

