//********************************************************************
//*** ResourcePool/oracle/oracleType.cc
//*** Copyright (c) 2003-2005 by Markus Winand <mws@fatalmind.com>
//*** $Id: oracleType.cc,v 1.10 2005/01/06 16:28:00 mws Exp $
//********************************************************************
/*
This file is part of ResourcePool.

ResourcePool is free software; you can redistribute it
and/or modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

ResourcePool is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied warranty
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ResourcePool; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
02111-1307  USA */

#ifndef INCLUDED_EXCEPTIONS_HH
#include "Exceptions/Exception.hh"
#endif

#ifndef RESOURCEPOOL_ORACLE_ORACLETYPE_HH
#include "oracleType.hh"
#endif

#include <limits>

namespace fatalmind {
namespace oracle {

template<typename T>
oracleTypeImpl<T>::oracleTypeImpl(const T& val)
: _val(val)
{
}

template<typename T>
oracleTypeImpl<T>::~oracleTypeImpl()
{
}

template<typename T>
void* oracleTypeImpl<T>::ptr() const
{
	return (void*)&_val;
}

template<typename T>
sb4 oracleTypeImpl<T>::size() const
{
	return sizeof(_val);
}

template<typename T>
ub2 oracleTypeImpl<T>::type() const
{
	return std::numeric_limits<T>::is_signed?SQLT_INT:SQLT_UIN;
}

/*
 * string specialization
 */

template<>
void* oracleTypeImpl<std::string>::ptr() const
{
	return (void*)_val.c_str();
}

template<>
sb4 oracleTypeImpl<std::string>::size() const
{
	return _val.length();
}

template<>
ub2 oracleTypeImpl<std::string>::type() const
{
	return SQLT_CHR;
}

/*
 * speicalizations for built in types
 */

template<>
ub2 oracleTypeImpl<float>::type() const
{
	return SQLT_FLT;
}

template<>
ub2 oracleTypeImpl<double>::type() const
{
	return SQLT_FLT;
}

/*
 * Instantiation
 */
template class oracleTypeImpl<short>;
template class oracleTypeImpl<int>;
template class oracleTypeImpl<long>;
template class oracleTypeImpl<long long>;
template class oracleTypeImpl<float>;
template class oracleTypeImpl<double>;
template class oracleTypeImpl<std::string>;


dvoid* oracleType::indptr() const {
	return (dvoid*)&ind;
}

bool oracleType::isnull() const {
	return (ind == -1);
}

void oracleType::setnull(bool nul) {
	if (nul) {
		ind = -1;
	} else {
		ind = 0;
	}
}

oracleType::~oracleType() {
}

}
}
